package v3d.editor;

import java.io.*;

/*
 * This is a class that implements a 3D vector and various
 * operations related to it.
 *
 */

class Vector3D
{
	public float x,y,z;

	Vector3D() {}

	Vector3D(float xa, float ya, float za) {
		this.x = xa;
		this.y = ya;
		this.z = za;
	}

	Vector3D(Vector3D vec) {
		this.x = vec.x;
		this.y = vec.y;
		this.z = vec.z;
	}

	public void addToThis(Vector3D vec) {
		this.x += vec.x;
		this.y += vec.y;
		this.z += vec.z;
	}

	public void copy(Vector3D vec) {
		this.x = vec.x;
		this.y = vec.y;
		this.z = vec.z;
	}

	public Vector3D add(Vector3D vec) {
		Vector3D tVec = new Vector3D();
		tVec.x = this.x + vec.x;
		tVec.x = this.y + vec.y;
		tVec.x = this.z + vec.z;
		return tVec;
	}

	public void subtractFromThis(Vector3D vec) {
		this.x -= vec.x;
		this.y -= vec.y;
		this.z -= vec.z;
	}

	public Vector3D subtract(Vector3D vec) {
		Vector3D tVec = new Vector3D();
		tVec.x = this.x - vec.x;
		tVec.y = this.y - vec.y;
		tVec.z = this.z - vec.z;
		return tVec;
	}

	public void multiplyThis(float f) {
		this.x *= f;
		this.y *= f;
		this.z *= f;
	}

	public Vector3D multiply(float f) {
		Vector3D tVec = new Vector3D();
		tVec.x = this.x * f;
		tVec.y = this.y * f;
		tVec.z = this.z * f;
		return tVec;
	}

	public void divideThis(float f) {
		this.x /= f;
		this.y /= f;
		this.z /= f;
	}

	public Vector3D divide(float f) {
		Vector3D tVec = new Vector3D();
		tVec.x = this.x / f;
		tVec.y = this.y / f;
		tVec.z = this.z / f;
		return tVec;
	}

	public void inverseThis() {
		this.x = -this.x;
		this.y = -this.y;
		this.z = -this.z;
	}

	public Vector3D inverse() {
		Vector3D tVec = new Vector3D();
		tVec.x = -this.x;
		tVec.y = -this.y;
		tVec.z = -this.z;
		return tVec;
	}

	public boolean isEqual(Vector3D vec) {
		if (this.x == vec.x && this.y == vec.y && this.z == vec.z)
			return true;
		return false;
	}

	public float dotProd(Vector3D vec) {
		return this.x * vec.x + this.y * vec.y + this.z * vec.z;
	}

	public Vector3D crossProd(Vector3D vec) {
		float a, b, c;
		a = y * vec.z - z * vec.y;
		b = -(x * vec.z - z * vec.x);
		c = x * vec.y - y * vec.x;
		return new Vector3D(a, b, c);
	}

	public float length() {
		return (float)Math.sqrt(lengthSq());
	}

	public float lengthSq() {
		return this.x * this.x + this.y * this.y + this.z * this.z;
	}

	public void normalizeThis() {
		float len = length();

		if (len != 0) {
			this.divideThis(len);
		}
	}
	
	public Vector3D normalize() {
		Vector3D tVec = new Vector3D(this);

		float len = length();

		if (len != 0) {
			tVec.divideThis(len);
		}

		return tVec;
	}

	public void print() {
		System.out.println("x: "+this.x+" y: "+this.y+" z: "+this.z);
	}

	// Vector is defined with a line starting with word "v". The values on the line define
	// the position of the vector (three floats).

	// Creates a string of the vector
	public String serialize() {
		String s = new String();
		s =	"v " +
			x + " " +
			y + " " +
			z + "\n";
		return s;
	}

	// Tries to read a vertex from the stream. Returns
	// true if the token was correct and data was read,
	// otherwise false.
	public boolean deserialize(StreamTokenizer st) throws IOException {
		if ("v".equals(st.sval)) {
			if (st.nextToken() == StreamTokenizer.TT_NUMBER) {
				x = (float)st.nval;
				if (st.nextToken() == StreamTokenizer.TT_NUMBER) {
					y = (float)st.nval;
					if (st.nextToken() == StreamTokenizer.TT_NUMBER)
						z = (float)st.nval;
				}
			}

			while (st.ttype != StreamTokenizer.TT_EOL &&
				st.ttype != StreamTokenizer.TT_EOF)
				st.nextToken();

			return true;
		}
		else {
			return false;
		}
	}

	// Creates a VRML string of the vector
	public String getVRML() {
		String s = new String();
		s =	x + " " +
			y + " " +
			(-z);
		return s;
	}

}

class Vector4D
{
	public float x,y,z,w;

	Vector4D() {}

	Vector4D(float x, float y, float z, float w) {
		this.x = x;
		this.y = y;
		this.z = z;
		this.w = w;
	}
}
